@@ -40,6 +40,7 @@ import threading
 import time
 
 from dbs.apis.dbsClient import DbsApi
+from RestClient.ErrorHandling.RestClientExceptions import HTTPError
 
 from Utils.Timers import timeFunction
 from WMComponent.DBS3Buffer.DBSBufferBlock import DBSBufferBlock
@@ -48,6 +49,7 @@ from WMCore.Algorithms.MiscAlgos import sortListByKey
 from WMCore.DAOFactory import DAOFactory
 from WMCore.Services.UUIDLib import makeUUID
 from WMCore.Services.WMStatsServer.WMStatsServer import WMStatsServer
+from WMCore.Services.DBS.DBSErrors import DBSError
 from WMCore.WMException import WMException
 from WMCore.WorkerThreads.BaseWorkerThread import BaseWorkerThread
 
@@ -91,25 +93,39 @@ def uploadWorker(workInput, results, dbsUrl, gzipEncoding=False):
             logging.info("About to call insert block for: %s", name)
             dbsApi.insertBulkBlock(blockDump=block)
             results.put({'name': name, 'success': "uploaded"})
-        except Exception as ex:
-            exString = str(getattr(ex, "body", ex))
-            if 'Block %s already exists' % name in exString:
+        except HTTPError as ex:
+            # when we will have new dbs3-client and fine granularity DBS Go server
+            # error codes then we should remove if/elif/else and replace it with
+            # DBSError(ex.body), e.g.
+            # dbsError = DBSError(ex.body)
+            # reason = dbsError.getReason()
+            # srvCode = dbsError.getServverCode()
+            # results.put({'name': name, 'success': 'error', 'error': reason, 'server_code': srvCode})
+
+            # legacy code which deal with DBS python server exceptions
+            dbsError = DBSError(ex.body)
+            reason = dbsError.getReason()
+            if 'Block %s already exists' % name in reason:
                 # Then this is probably a duplicate
                 # Ignore this for now
                 logging.warning("Block %s already exists. Marking it as uploaded.", name)
-                logging.debug("Exception: %s", exString)
+                logging.debug("Exception: %s", reason)
                 results.put({'name': name, 'success': "uploaded"})
-            elif 'Missing data when inserting to dataset_parents' in exString:
+            # python server error produces Missing message, while
+            # Go based server produces unable to find dataset_id message
+            elif 'Missing data when inserting to dataset_parents' in reason or \
+                'unable to find dataset_id' in reason:
                 msg = "Parent dataset is not inserted yet for block %s." % name
                 logging.warning(msg)
                 results.put({'name': name, 'success': "error", 'error': msg})
             else:
-                reason = parseDBSException(exString)
-                msg = "Error trying to process block %s through DBS. Error: %s" % (name, reason)
-                logging.exception(msg)
-                logging.debug("block info: %s \n", block)
+                msg = f"Error trying to process block {name} through DBS. Reason: {reason}"
+                logging.error(msg)
                 results.put({'name': name, 'success': "error", 'error': msg})
-
+        except Exception as ex:
+            msg = f"Hit a general exception while inserting block {name. Error: {str(ex}}"
+            logging.exception(msg)
+            results.put({'name': name, 'success': "error", 'error': msg})
     return
 
 
@@ -128,31 +144,6 @@ def parseDBSException(exBodyString):
         return exBodyString
 
 
-def isPassiveError(exceptionObj):
-    """
-    This function will parse the exception object and report whether
-    the error message corresponds to a soft or hard error (hard errors
-    are supposed to let the component crash).
-    :param exceptionObj: any exception object
-    :return: True if it's a soft error, False otherwise
-    """
-    passException = True
-    passiveErrorMsg = ['Service Unavailable', 'Service Temporarily Unavailable',
-                       'Proxy Error', 'Error reading from remote server',
-                       'Connection refused', 'timed out', 'Could not resolve',
-                       'OpenSSL SSL_connect: SSL_ERROR_SYSCALL']
-
-    excReason = getattr(exceptionObj, 'reason', '')
-    for passiveMsg in passiveErrorMsg:
-        if passiveMsg in excReason:
-            break
-        elif passiveMsg in str(exceptionObj):
-            break
-    else:
-        passException = False
-    return passException
-
-
 class DBSUploadException(WMException):
     """
     Holds the exception info for
@@ -357,17 +348,12 @@ class DBSUploadPoller(BaseWorkerThread):
         try:
             self.datasetParentageCache = self.wmstatsServerSvc.getChildParentDatasetMap()
         except Exception as ex:
+            success = False
             excReason = getattr(ex, 'reason', '')
             errorMsg = 'Failed to fetch parentage map from WMStats, skipping this cycle. '
-            errorMsg += 'Exception: {}. Reason: {}. Error: {}. '.format(type(ex).__name__,
-                                                                        excReason, str(ex))
-            if isPassiveError(ex):
-                logging.warning(errorMsg)
-            else:
-                errorMsg += 'Hit a terminal exception in DBSUploadPoller.'
-                raise DBSUploadException(errorMsg) from None
+            errorMsg += 'Reason: {}. Error: {}. '.format(excReason, str(ex))
+            logging.error(errorMsg)
             myThread.logdbClient.post("DBS3Upload_parentMap", errorMsg, "warning")
-            success = False
         else:
             myThread.logdbClient.delete("DBS3Upload_parentMap", "warning", this_thread=True)
 
@@ -392,7 +378,7 @@ class DBSUploadPoller(BaseWorkerThread):
         # Now load the blocks
         try:
             loadedBlocks = self.dbsUtil.loadBlocks(blocksToLoad)
-            logging.info("Loaded %d blocks.", len(loadedBlocks))
+            logging.info("Loaded %d blocks from the database.", len(loadedBlocks))
         except WMException:
             raise
         except Exception as ex:
@@ -414,6 +400,9 @@ class DBSUploadPoller(BaseWorkerThread):
             block.FillFromDBSBuffer(blockInfo)
             blockname = block.getName()
 
+            # Now we load the dataset information
+            self.setDatasetInfo(block)
+
             # Now we have to load files...
             try:
                 files = self.dbsUtil.loadFilesByBlock(blockname=blockname)
@@ -573,6 +562,8 @@ class DBSUploadPoller(BaseWorkerThread):
         newBlock = DBSBufferBlock(name=blockname,
                                   location=location,
                                   datasetpath=datasetpath)
+        # Now we load the dataset information
+        self.setDatasetInfo(newBlock)
 
         parent = self.datasetParentageCache.get(datasetpath)
         if parent:
@@ -582,6 +573,24 @@ class DBSUploadPoller(BaseWorkerThread):
         self.blockCache[blockname] = newBlock
         return newBlock
 
+    def setDatasetInfo(self, blockObj):
+        """
+        Given a block object, look up its dataset and set the necessary
+        data structure in the object
+        :param blockObj: a DBSBufferBlock object
+        :return: None, the object is updated here
+        """
+        try:
+            dsetInfo = self.dbsUtil.loadDataset(blockObj.getDatasetPath())
+            blockObj.setDataset(datasetName=dsetInfo['path'],
+                                primaryType=self.primaryDatasetType,
+                                datasetType=dsetInfo['valid_status'],
+                                prep_id=dsetInfo['prep_id'])
+        except Exception as ex:
+            msg = f"Unhandled exception while loading/setting dataset for block: {blockObj.getName()}. "
+            msg += f"Details: {str(ex)}"
+            raise DBSUploadException(msg) from None
+
     def inputBlocks(self):
         """
         _inputBlocks_
@@ -693,14 +702,7 @@ class DBSUploadPoller(BaseWorkerThread):
                 # What are we doing?
                 logging.debug("Skipping empty block")
                 continue
-            if block.getDataset() is None:
-                # Then we have to fix the dataset
-                dbsFile = block.files[0]
-                block.setDataset(datasetName=dbsFile['datasetPath'],
-                                 primaryType=self.primaryDatasetType,
-                                 datasetType=self.datasetType,
-                                 physicsGroup=dbsFile.get('physicsGroup', None),
-                                 prep_id=dbsFile.get('prep_id', None))
+
             logging.debug("Found block %s in blocks", block.getName())
             block.setPhysicsGroup(group=self.physicsGroup)
 
